package util;

/**
 * Generic, vararg version of a lambda command, where the input parameter and return types can be 
 * explicitly defined.  The input parameter is a vararg, allowing for zero or multiple parameters
 * to be passed.   Note that even if the concrete implementation does not use any input parameters, 
 * an input parameter type must be define--usually just "Object" in that case.  Likewise for the return 
 * type.  Concrete implementations should return "null" if no return value is otherwise generated. 
 * @author Stephen Wong (c) 2010
 *
 * @param <R>  The type of the return value
 * @param <P>  The type of the vararg input parameters.
 */
public interface ILambda<R, P> {
	/**
	 * The method to invoke this command's behavior.
	 * @param params  The vararg input parameters the command uses.
	 * @return  The return value of the command.
	 */
	public R apply(P...params);
}
